# Generated by Django 2.0.4 on 2018-07-19 18:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('deeppages', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='page',
            name='changed_at',
            field=models.DateTimeField(auto_now=True, verbose_name='Changed At'),
        ),
        migrations.AlterField(
            model_name='page',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Created At'),
        ),
    ]



from django.views.generic import TemplateView
from django.http import (
    HttpResponse,
    HttpResponseBadRequest,
    HttpResponseNotFound,
)

from .exceptions import InvalidPathException, PageNotFoundException
from .utils import get_page_by_path

import logging

logger = logging.getLogger(__name__)


class PageView(TemplateView):
    def render_to_response(self, context, **response_kwargs):
        try:
            rendered_page = get_page_by_path(self, self.request, logger)

        except InvalidPathException:
            return HttpResponseBadRequest()

        except PageNotFoundException:
            return HttpResponseNotFound(
                'Page ({}) not found'.format(self.request.get_full_path()))

        else:
            return HttpResponse(rendered_page)



from django.utils.deprecation import MiddlewareMixin
from django.http.response import HttpResponse

from .utils import get_page_by_path

import logging

logger = logging.getLogger(__name__)


class DeepPageTemplateRendererMiddleware(MiddlewareMixin):
    '''
        The goal of this middleware is find the respective page by URL, render
        the page content and return it.
    '''

    # TODO: check where is accessing request.POST/body and change the code.
    # Also, create all possible tests before push new version. Maybe this
    # should be a version bump to 1.0-stable.
    def process_request(self, request):
        '''
            Look for Deep Page's PATH pattern and return a rendered template
        '''

        # DeepPages must work ONLY if actual response has status_code == 404
        # (Not found). Otherwise, there is no reason to interfere into normal
        # page response behavior.
        if self.get_response(request).status_code == 404:
            rendered_page = get_page_by_path(self, request, logger)

            if rendered_page:
                return HttpResponse(rendered_page)



from django.template import Template, Context
from django.utils.deprecation import MiddlewareMixin
from django.conf import settings
from django.db.models import Q
from django.urls import reverse, NoReverseMatch
from django.core.exceptions import ObjectDoesNotExist

from .signals import page_found, page_not_found, page_requested
from .exceptions import InvalidPathException, PageNotFoundException
from .models import Page

import re


def normalize_path(path):
    ''' Remove duplicated slashes and reverse mode with/wo slash in the end '''
    from .urls import get_deeppages_path

    new_path = re.sub('[\/]{2,}', '/', path)

    try:
        # check if deeppages' path isn't the root path
        deeppages_path = reverse('deeppages:{}'.format(get_deeppages_path()))
    except NoReverseMatch:
        pass
    else:
        if deeppages_path != '/':
            if new_path.startswith(deeppages_path):
                new_path = new_path.replace(deeppages_path, '')
                if not new_path.startswith('/'):
                    new_path = '/{}'.format(new_path)

    return new_path[:-1] if new_path.endswith('/') else '{}/'.format(new_path)


def render_content(content, context):
    ''' Render page content '''
    ctx = Context(context or {})
    return Template(content).render(ctx)


def render_page(page, context, callback):
    ''' Render page '''
    if callback:
        page_content = callback(page, context)
    else:
        page_content = page.content

    return render_content(page_content, context)


def render_requested_page_content(sender, request, page):
    ''' Render page requested by Middleware or PageView '''
    content = page.content
    ctx = {'request': request}

    page_found.send_robust(
        sender=sender.__class__,
        page=page,
        path=page.path,
        request=request,
        content=content,
        context=ctx)

    # So, if content and/or context was changed inside the signal receiver,
    # we'll render with the new values.
    return render_content(content, ctx)


def is_acceptable_file_type(path):
    ''' Only text-based content can be accepted, any other will be ignored. '''
    filename = path.strip('/').split('/')[-1]

    accepted_exts = ['.html', '.htm', '.css', '.js', '.svg', '.txt']
    max_ext_len = max(map(len, accepted_exts))

    try:
        has_extension = filename.index('.') >= (len(filename) - max_ext_len)
    except ValueError:
        has_extension = False

    is_accepted = not has_extension or len([a for a in accepted_exts
                                            if filename.endswith(a)]) > 0

    return is_accepted


def get_page_by_path(sender, request, logger):
    '''
        Get page by path and return a rendered and processed template.

        Arguments:

            sender -- object sender
            request -- WSGIRequest object
            logger -- logger instance

        Also, three robust signals can be dispatched from here:
            1. page_requested (after a page request, ha!)
            2. page_not_found (for non-existent pages! O'really?)
            3. and, mainly, page_found (When a page exists AND is active! Ha!
               Could you imagine that?)

        Both signals: 'page_request' and 'page_not_found' these keyword
        arguments will be received: 'path' and 'request'.

        For 'page_found':
            - path: the path (URL) requested
            - page: a deeppages.models.Page() model's instance that was found
                    by its PATH
            - request: WSGIRequest object
            - context: a context dictionary (with request inside)
            - content: the page content (you can change it as you wish)

        In case of 'page_not_found', after robust signal callback has been
        returned, Django's will follow its normal flow.

        ps.: if settings.DEBUG is True, you can handle some logs for debug
             purposes.
    '''
    path = normalize_path(request.path)

    if not is_acceptable_file_type(path):
        return

    if settings.DEBUG and logger:
        logger.debug('DeepPage Path Requested: [{}]'.format(path))

    # dispatch page requested signal
    page_requested.send_robust(
        sender=sender.__class__, path=path, request=request)

    if not path:
        # Is called from an instance subclass of TemplateView ?
        if issubclass(sender.__class__, MiddlewareMixin):
            return
        else:
            raise InvalidPathException

    try:
        # try to get page directly
        page = Page.objects.exclude(is_active=False).get(
            Q(path__iexact=path) | Q(path__iexact=request.path))

    except Page.DoesNotExist:
        if settings.DEBUG and logger:
            logger.exception('DeepPage Not Found: [{}]'.format(path))

        page_not_found.send_robust(
            sender=sender.__class__,
            path=path,
            request=request)

        if issubclass(sender.__class__, MiddlewareMixin):
            return
        else:
            raise PageNotFoundException
    else:
        return render_requested_page_content(sender, request, page)


def get_page_by_name(name, context=None, callback=None):
    '''
        Get page by its name and render it.

        Arguments:

            name -- Page name

        Keyword arguments:

            context -- dictionary with additional key/values that
            will be used for page content rendering (default: None)

            callback -- callback function - will be called before render the
            page content (default: None)
    '''
    if not name:
        return

    try:
        # try to get page directly
        page = Page.objects.exclude(is_active=False).get(name__iexact=name)

    except ObjectDoesNotExist:
        return

    else:
        return render_page(page, context, callback)



from django.dispatch import Signal


page_requested = Signal(providing_args=['path', 'request'])
page_not_found = Signal(providing_args=['path', 'request'])
page_found = Signal(providing_args=[
    'path', 'request', 'page', 'content', 'context'])



from django.contrib import admin
from django.forms import Textarea
from django.db import models
from deeppages.models import Page


@admin.register(Page)
class PageAdmin(admin.ModelAdmin):
    formfield_overrides = {
        models.TextField: {
            'widget': Textarea(attrs={'rows': 20, 'style': 'width: 99%;'})
        },
    }



from django.test import TestCase
# from model_mommy import mommy


class PageTestCase(TestCase):
    def test_deep_page_settings(self):
        pass

    def test_create_deep_page(self):
        pass

    def test_create_duplicated_path_pages(self):
        pass

    def test_request_deep_page(self):
        pass



from django.conf.urls import url
from django.conf import settings

from .views import PageView


def get_deeppages_path():
    try:
        path = settings.DEEP_PAGES_URL_NAME
    except AttributeError:
        path = '__deep_page_view__'

    return path


urlpatterns = [
    url('', PageView.as_view(), name=get_deeppages_path()),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-03-21 04:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Page',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='This name will be used to identify the page. Ex.: my page name', max_length=50, unique=True)),
                ('path', models.CharField(help_text='Type the absolute path. Ex.: /my-page-name/', max_length=100)),
                ('title', models.CharField(blank=True, default=None, max_length=100, null=True)),
                ('content', models.TextField(blank=True, default=None, null=True)),
                ('is_active', models.BooleanField(default=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name=b'Created At')),
                ('changed_at', models.DateTimeField(auto_now=True, verbose_name=b'Changed At')),
            ],
            options={
                'ordering': ('name',),
                'get_latest_by': 'changed_at',
                'verbose_name': 'Page',
                'verbose_name_plural': 'Pages',
            },
        ),
    ]



from setuptools import setup, find_packages


setup(
    name='deeppages',
    version='0.1.8',
    description="Django's database stored web content processor",
    long_description = open('README.md').read(),
    url='https://github.com/ricardofalasca/deep-pages',
    author='Ricardo Falasca',
    author_email='ricardo@falasca.com.br',
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
    license='MIT',
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
    ]
)



from django.utils.translation import ugettext_lazy as _

from django.db import models
from django.urls import resolve, reverse, Resolver404


class Page(models.Model):
    ''' The unique model for Deep Pages '''

    name = models.CharField(
        max_length=50,
        unique=True,
        help_text=_('This name will be used to identify the page. Ex.: my '
                    'page name'))

    path = models.CharField(
        max_length=100,
        help_text=_('Type the absolute path. Ex.: /my-page-name/'))

    title = models.CharField(
        max_length=100,
        default=None,
        blank=True,
        null=True)

    content = models.TextField(null=True, blank=True, default=None)
    is_active = models.BooleanField(default=True)
    created_at = models.DateTimeField('Created At', auto_now_add=True)
    changed_at = models.DateTimeField('Changed At', auto_now=True)

    class Meta:
        verbose_name = _('Page')
        verbose_name_plural = _('Pages')
        ordering = ('name',)
        get_latest_by = 'changed_at'

    def _validate_path(self):
        try:
            resolve(self.path)

        except Resolver404:
            return True

        else:
            return self.path != reverse('deeppages:__deep_page_view__')

    def __unicode__(self):
        return self.name

    def __str__(self):
        return self.name

    def render(self, context=None, callback=None):
        from .utils import render_content
        ctx = context or {}
        return render_content(self.content, ctx)

    def save(self, *args, **kwargs):
        if not self._validate_path():
            print(_('This PATH is already being used. Try a different path.'))
            return False
        else:
            return super(Page, self).save(*args, **kwargs)
